#include <BF_Parameters.h>
#include <functions.h>
#include <SensorDefinitions.h>

extern char CommandArray[5];					//Contains sensor commands
extern char SensorDataArray[5];					//Contains register-data received from the sensor
extern char SPITransferStatus;					//Slow SPI transfer status
extern char SPITransferCount;					//Command / Data transfer count - Slow SPI mode
extern char SensorRequest;						//0: Sensor request; 1: No sensor request
extern char N;									//Maximun command / Data transfer count - Slow SPI mode
extern char Data[FN][233][4];					//Raw Sensor data
extern char DataProcessed[FN][8][232];			//Processed sensor data
extern char COREorDMA;							//0: Core; 1: DMA
extern char ProcessData;						//0: Request for process data; 1: No request for process data
unsigned short DmaDescriptor[FN+1][8];			//DMA Descriptors

/*
Function Definitions
*/

//--------------------------------------------------------------------------//
// Function:	PLLSetupBF													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up the VCO, Core Clock and System clock  //
//				for BF. These clock values are defined in BF_Parameters.h	//
//--------------------------------------------------------------------------//
void PLLSetupBF()
{
    *pSIC_IWR = 0x1;
	*pPLL_CTL = SET_MSEL(VCO / CLKin);
	ssync();
	idle();
	*pPLL_DIV = SET_SSEL(VCO / SystemCLK) | ((VCO_CoreCLK)&0x30);
	asm("ssync;");
}

//--------------------------------------------------------------------------//
// Function:	SetupInterruptsBF											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up the interrupts for SPI and PF4 (IRQ). //
//				The SPI interrupts are mapped to IVG10 and the PF4 			//
//				are mapped to IVG12											//
//--------------------------------------------------------------------------//
void SetupInterruptsBF()
{
    register_handler(ik_ivg10,IsrSPI);
    *pSIC_IMASK |= DMA5_IRQ;
    
    *pFIO_INEN = 0x10;
    *pFIO_EDGE = 0x10;
    *pFIO_POLAR = 0x10;
    *pFIO_MASKA_S = 0x10;
    
    register_handler(ik_ivg12, IsrIRQ);
    *pSIC_IMASK |= PFA_IRQ;
}

//--------------------------------------------------------------------------//
// Function:	SetUpSlaveSelects											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures PF1 and PF2 as output flags and    //
//				drives them logic HIGH.										//
//--------------------------------------------------------------------------//
void SetUpSlaveSelects()
{
    *pFIO_DIR |= PF1; 							//Configure PF1 as output - SSS/
    *pFIO_FLAG_S = PF1;							//Drive PF1 Logic HIGH - SSS/ de-asserted state
    *pFIO_DIR |= PF2;							//Configure PF2 as output - FSS/
    *pFIO_FLAG_S = PF2;							//Drive PF2 Logic HIGH - FSS/ de-asserted state
    DelayMin1SckCycle();
}

//--------------------------------------------------------------------------//
// Function:	ResetSensor													//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function configures PF3 as an output and drives it     //
//				logic HIGH for at least 10usec. It also provides delay in   //
//				order to meet Trstsu value									//
//--------------------------------------------------------------------------//
void ResetSensor()
{
    int i;
    *pFIO_DIR |= PF3; 							//Configure PF3 as output
    *pFIO_FLAG_S = PF3;							//Drive PF3 Logic HIGH - Assert RESET
    
    for(i=0; i<0xFFF; i++)						//Delay of at least 10usec
    {
        asm("nop;");
    }
    
    *pFIO_FLAG_C = PF3;							//Drive PF3 Logic LOW - De-assert RESET
    
    //This is to take care of Trstsu
    DelayMin1SckCycle();
}

//--------------------------------------------------------------------------//
// Function:	SetUpAndEnableSlowSPI										//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up SPI for slow mode. It drives SSS/ 	//
//				active to enable the slow SPI of the sensor. This function  //
//				provide the delay Tsssu										//	
//--------------------------------------------------------------------------//
void SetUpAndEnableSlowSPI()
{
    *pSPI_CTL = 0;
    *pSPI_CTL = CPOL | MSTR | SZ | GM | CPHA;				//Configure BF SPI as Master, CPOL=1, CPHA = 1
    *pSPI_BAUD = SystemCLK / SlowSPIBaud / 2;
    *pFIO_FLAG_C = PF1;										//Drive PF1 Logic LOW - Assert SSS/
    
    DelayMin1SckCycle();
    *pSPI_CTL |= SPE;
}

//--------------------------------------------------------------------------//
// Function:	SetUpAndEnableFastSPI										//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up SPI for fast mode. It drives FSS/ 	//
//				active to enable the fast SPI mode of the sensor.			//
//--------------------------------------------------------------------------//
void SetUpAndEnableFastSPI()
{
    *pSPI_CTL = 0;
    *pSPI_CTL = CPOL | MSTR | SZ | GM | CPHA | RDBR_DMA;	//Configure BF SPI as Master, CPOL=1, CPHA = 1, DMA on RX read
    *pSPI_BAUD = SystemCLK / FastSPIBaud / 2;
    *pFIO_FLAG_C = PF2;										//Drive PF2 Logic LOW - Assert FSS/
    *pSPI_CTL |= SPE;
}

//--------------------------------------------------------------------------//
// Function:	SetupDMA												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up SPI DMA parameters for data 
//				acquisition from sensor.									//
//--------------------------------------------------------------------------//
void SetupDMA()
{
    int i = 0;
    DmaDescriptor[0][0] = (unsigned short)(DmaDescriptor[1]);
    DmaDescriptor[0][1] = ((unsigned short) Data[0]);
    DmaDescriptor[0][2] = ((unsigned short)((unsigned int)Data[0] >> 16) & 0xffff) ;
    DmaDescriptor[0][3] = FLOW_SMALL | NDSIZE_8 | WNR | DMAEN;
    DmaDescriptor[0][4] = 5;
    DmaDescriptor[0][5] = 0x1;
    
    for(i=0;i<FN;i++)
    {
        DmaDescriptor[i+1][0] = ((unsigned short) DmaDescriptor[i+2]);
	    DmaDescriptor[i+1][1] = ((unsigned short) Data[i]);
	    DmaDescriptor[i+1][2] = ((unsigned short)((unsigned int)Data[i] >> 16) & 0xffff) ;
	    DmaDescriptor[i+1][3] = FLOW_SMALL | NDSIZE_8 | DMA2D | WNR | DMAEN;
	    DmaDescriptor[i+1][4] = 4;
	    DmaDescriptor[i+1][5] = 0x1;
	    DmaDescriptor[i+1][6] = 233;
	    DmaDescriptor[i+1][7] = 0x1;
    }
        
    DmaDescriptor[FN][0] = ((unsigned short) Data[FN-1]);
    DmaDescriptor[FN][1] = ((unsigned short) Data[FN-1]);
    DmaDescriptor[FN][2] = ((unsigned short)((unsigned int)Data[FN-1] >> 16) & 0xffff) ;
    DmaDescriptor[FN][3] = DI_EN | DMA2D | WNR | DMAEN;
    DmaDescriptor[FN][4] = 4;
    DmaDescriptor[FN][5] = 0x1;
    DmaDescriptor[FN][6] = 233;
    DmaDescriptor[FN][7] = 0x1;
    
    *pDMA5_NEXT_DESC_PTR = (unsigned int *)DmaDescriptor[0] ;
	*pDMA5_CONFIG = FLOW_SMALL | NDSIZE_6  | WNR | DMAEN;
	COREorDMA = 1;
}

//--------------------------------------------------------------------------//
// Function:	DisableSlowSPI												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This fucntion checks for RX buffer full status and SPI 		//
//				transfer complete status and disables SPI. It de-asserts 	//
//				slow (SSS/) SPI select signal								//
//--------------------------------------------------------------------------//
void DisableSlowSPI()
{
 	while(!(*pSPI_STAT & 0x0020));				//Check for Rx Buffer Full
 	while(!(*pSPI_STAT & 0x0001));				//Check for SPIF
 	DelayMin1SckCycle();
 	*pSPI_CTL &= ~SPE;							//Disable SPI
 	SensorDataArray[0] = *pSPI_RDBR;			//Clears the SPI RX buffer full status
 	*pFIO_FLAG_S = PF1;							//Drive PF1 Logic HIGH - de-assert SSS/
}

//--------------------------------------------------------------------------//
// Function:	DisableFastSPI												//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This fucntion disables SPI and de-asserts slow (SSS/) 		//
// SPI select signal														//
//--------------------------------------------------------------------------//
void DisableFastSPI()
{
    DelayMin1SckCycle();
    *pSPI_CTL &= ~SPE;
    *pDMA5_CONFIG = 0x0;
    *pFIO_FLAG_S = PF2;							//Drive PF2 Logic HIGH - de-assert FSS/
}

//--------------------------------------------------------------------------//
// Function:	WriteSensorRegister											//
//																			//
// Parameters:	CommandAndAddress, value									//
//																			//
// Return:		None														//
//																			//
// Description:	Command and data to be written to the sensor is passed to	//
//				this fuction. These values are initialized in to a global	//
//				command array. It calls functions required to set-up SPI 	//
//				and perform the transfer.									//
//--------------------------------------------------------------------------//
void WriteSensorRegister(int CommandAndAddress, int value)
{
    SetUpCommandArray(CommandAndAddress, value);
    SetUpAndEnableSlowSPI();
    SPITransferCount = 0;
    SPITransferStatus = 1;
    *pSPI_TDBR = CommandArray[0];
    SensorDataArray[0] = *pSPI_RDBR;
    while (SPITransferStatus);
}


//--------------------------------------------------------------------------//
// Function:	ReadSensorRegister											//
//																			//
// Parameters:	CommandAndAddress											//
//																			//
// Return:		Boolean														//
//																			//
// Description:	Read sensor register command is passed to this function.	//
//				It calls functions required to set-up SPI and perform the   //
//				transfer.													//
//--------------------------------------------------------------------------//
bool ReadSensorRegister(int CommandAndAddress)
{
    SetUpCommandArray(CommandAndAddress, 0x0);
    SetUpAndEnableSlowSPI();
    SPITransferStatus = 1;
    SPITransferCount = 0;
    *pSPI_TDBR = CommandArray[0];
    SensorDataArray[0] = *pSPI_RDBR;
    while (SPITransferStatus);
    if(CommandArray[0] & StatusRead)
    {
        if(StatusRead & READERR)
        {
            return 1;
        }
        else
        {
            return 0;
        }
            
    }
    else
    {
        return 0;
    }
}


//--------------------------------------------------------------------------//
// Function:	SetUpCommandArray											//
//																			//
// Parameters:	CommandAndAddress, value									//
//																			//
// Return:		None														//
//																			//
// Description:	This function sets up the global command array.				//
//--------------------------------------------------------------------------//
void SetUpCommandArray(int CommandAndAddress, int value)
{
    CommandArray[0] = CommandAndAddress;
    CommandArray[1] = value;
    N = 1; 										//SPITransferCount limit 
   
    if(CommandAndAddress == NavigationRead)
    {
        CommandArray[2] = 0;
        CommandArray[3] = 0;
        CommandArray[4] = 0;
        N = 4; 									//SPITransferCount limit 
    }
}

//--------------------------------------------------------------------------//
// Function:	DelayMin1SckCycle											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function provides a delay of > 1 slow SCK cycle.		//
//--------------------------------------------------------------------------//
void DelayMin1SckCycle()
{
    int i;
    for(i=0; i<0x277; i++)						
    {
        asm("nop;");
    }
}

//--------------------------------------------------------------------------//
// Function:	RearrangeData											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function re-arranges the raw pixel values in the array	//
//				Data and strores it into the array DataProcessed. The raw   //
//				values are re-arranged and a transpose of the array is taken//
//				It then does a stretching of gray level values from 16 to   //
//				256 levels - so the display is better.						//
//--------------------------------------------------------------------------//
void RearrangeData()
{
    int  temp1, temp2, i,j,k;
    for(k=0; k<FN; k++)
    {
        for(i=1; i<233; i++)
        {
            for(j=0; j<4; j++)
            {
                temp1 = Data[k][i][j] & 0xF;
                temp2 = Data[k][i][j] & 0xF0;
                temp2 = temp2 >> 4;
                /*temp1 = temp1 << 4;
                Data[k][i][j] = 0;
                Data[k][i][j] = temp1 & 0xF0;
                Data[k][i][j] = Data[k][i][j] | (temp2 & 0xF);
                DataProcessed[k][j][i] = Data[k][i][j];*/
                DataProcessed[k][(j*2)][i-1] = (temp1*0xF);
                DataProcessed[k][((j*2)+1)][i-1] = (temp2*0xF);
            }
        }
        
    }
}
